#ifndef _SPEECH_COMPARISON_H_
#define _SPEECH_COMPARISON_H_

#include <linux/limits.h>

#include <pocketsphinx.h>

#include <libsvm/svm.h>

#include "lsc_error.h"

#define MAX_LETTERS_IN_WORD		 32
#define MAX_WORDS_IN_SENTENCE		 64
#define MAX_FILENAME_LENGTH		256

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _ERR_FACTOR {
	int accentLevel[2];     // [0]: Benchmark's, [1]: Sample's
	int toneLevel[2];       // [0]: Benchmark's, [1]: Sample's
	int bMispronounceCand;  // 0: initial    1: correct    2: wrong
	int bFounded;           // ps_align_find  0: not   1: yes
} err_fact;

typedef struct _ALGN {
	int start, dur, cipid, score;
	float subs, insdel;
} stAlgn;

typedef struct _SALIENCY {
	float nInitial;
	float nFinal;
	float nMain;
	float nMainPos;
	float nMean;
	int pitchLevel;
} salience;

typedef struct _INTERVALWORD {
	char text[MAX_LETTERS_IN_WORD]; // word
	float xmin, xmax;       // time boundary of Benchmark
	int mfc_min, mfc_max;   // Benchmark's aligned boundary MFCC
	float aligned_min;      // start time of Sample's word
	float aligned_max;      // end time of Sample's word
	int aligned_mfc_min, aligned_mfc_max;   // Sample's ps_aligned boundary MFCC
	int dtw_mfc_min, dtw_mfc_max;   // Sample's dtw_aligned boundary MFCC
	int isFindWord;         // 0: no forced-aligned, 1: forced-aligned, 2: kws-spotting aligned
	err_fact factor;        // error factors
	int bfMatch;            // Is this word correct?
	salience sTones[2];     // tone
} intervalWord;

typedef struct _ALIGN_RESULT {
	int nWordCount;
	intervalWord *ptrWords;

	char data_dir[PATH_MAX];
	char model_dir[PATH_MAX];
	char dict_name[PATH_MAX];

	struct svm_model *smodel;

	int jsgf_word;
	char jsgf_tmp[PATH_MAX];

	FILE *result;
	size_t rsize;
	char *rptr;

	int g_nMfccDim0;
	int g_nMfccDim1;
	int g_nStdFrame;
	int g_nUttFrame;
	int g_nSubFrame;
	char g_strGrammar[1000];

	int16 *g_StdWavBuffer;
	int16 *g_UttWavBuffer;
	mfcc_t **g_mfccMat0;
	mfcc_t **g_mfccMat1;
	stAlgn *g_stAlgn;

	int max_nr_attr;
	int predict_probability;
} ad_align_result;

#pragma GCC visibility push(default)

int parse_cmd_ln(int cmdNum, char *cmds[]);
ad_align_result *ad_new(void);
int ad_init(ad_align_result *ad_words, const char *data_dir,
	    const char *tmp_dir, const char *labfname);
void ad_destroy(ad_align_result *ad_words);
int ad_execute(ad_align_result *ad_words);
void ad_get_results(ad_align_result *ad_words);
void ad_print_results(ad_align_result *ad_words);
int ad_load_wavs(ad_align_result *ad_words, const char *benchmark_wav,
		 const char *sample_wav);
void ad_free_wavs(ad_align_result *ad_words);

#pragma GCC visibility pop

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif
